@@ -174,7 +174,61 @@ class FieldQuerySource(object):
         """
         return QueryField(self.fieldname, 'is', values, target=self.target)
 
-    # FIXME - add "is_descendant" and "is_or_is_descendant"
+    def is_descendant(self, categories):
+        """Create a query for field values which are categories which are
+        descendants of one of the given categories.
+
+        A document will match if at least one of the stored values for the
+        field exactly matches a descendant of the given categories.
+
+        This query type is available only for "cat" field types.
+
+        :param categories: A container holding the categories to search for.
+               As a special case, if a string is supplied, this is equivalent
+               to supplying a container holding that string.
+
+        :example:
+
+            Search for documents in which the "tag" field is a descendant of
+            a value of "cheese"
+
+            >>> query = coll.field.tag.is_descendant('cheese')
+
+            or, equivalently:
+
+            >>> query = coll.field.tag.is_descendant(['cheese'])
+
+        """
+        return QueryField(self.fieldname, 'is_descendant', categories,
+                          target=self.target)
+
+    def is_or_is_descendant(self, categories):
+        """Create a query for field values which are categories which are
+        descendants of one of the given categories.
+
+        A document will match if at least one of the stored values for the
+        field exactly matches a descendant of the given categories.
+
+        This query type is available only for "cat" field types.
+
+        :param categories: A container holding the categories to search for.
+               As a special case, if a string is supplied, this is equivalent
+               to supplying a container holding that string.
+
+        :example:
+
+            Search for documents in which the "tag" field has a value of
+            "cheese", or has a value which is a descendant of "cheese".
+
+            >>> query = coll.field.tag.is_or_is_descendant('cheese')
+
+            or, equivalently:
+
+            >>> query = coll.field.tag.is_or_is_descendant(['cheese'])
+
+        """
+        return QueryField(self.fieldname, 'is_or_is_descendant', categories,
+                          target=self.target)
 
     def __eq__(self, value):
         """Create a query for fields which exactly match the given value.
@@ -579,6 +633,19 @@ class Collection(QueryTarget):
                           .expect_status(201)
                           .json.get('checkid'))
 
+    def taxonomies(self):
+        """Get a list of the taxonomy names.
+
+        """
+        path = self._basepath + "/taxonomy"
+        return self._resource.get(path).expect_status(200).json
+
+    def taxonomy(self, taxonomy_name):
+        """Access a taxonomy, for getting and setting its hierarchy.
+
+        """
+        return Taxonomy(self, taxonomy_name)
+
     def delete(self):
         """Delete the entire collection.
 
@@ -693,3 +760,64 @@ class CheckPoint(object):
             # the checkpoint, so we need to sleep to avoid using lots of CPU.
             import time
             time.sleep(1)
+
+class Taxonomy(object):
+    """A taxnonmy; a hierarchy of category relationships.
+
+    A collection may have many taxonomies, each identified by a name.  Each
+    taxonomy contains a set of categories, and a tree of parent-child
+    relationships (or, to use the correct mathematical terminology, a forest.
+    ie, there may be many disjoint trees of parent-child relationships).
+
+    This class allows the relationships in a taxonomy to be obtained and
+    modified.
+
+    """
+    def __init__(self, collection, taxonomy_name):
+        self._basepath = collection._basepath + '/taxonomy/' + taxonomy_name
+        self._resource = collection._resource
+
+    def all(self):
+        """Get details about the entire set of categories in the taxonomy.
+
+        This returns a dict, keyed by category ID, in which each each value is
+        a list of parent category IDs.
+
+        Raises ResourceNotFound if the collection or taxonomy are not found.
+
+        """
+        return self._resource.get(self._basepath).expect_status(200).json
+
+    def add_parent(self, category, parent):
+        """Add a parent to a category.
+
+        Creates the collection, taxononmy, category and the parent, if
+        necessary.
+
+        """
+        return self._resource.put(self._basepath + '/id/' + category +
+                                  '/parent/' + parent).expect_status(202).json
+
+    def remove_parent(self, category, parent):
+        """Remove a parent from a category.
+
+        Creates the collection and taxononmy if they don't already exist.
+
+        """
+        return self._resource.delete(self._basepath + '/id/' + category +
+                                '/parent/' + parent).expect_status(202).json
+
+    def remove_category(self, category):
+        """Remove a category.
+
+        Creates the collection and taxononmy if they don't already exist.
+
+        """
+        return self._resource.delete(self._basepath + '/id/' + category) \
+            .expect_status(202).json
+
+    def remove(self):
+        """Remove this entire taxonomy.
+
+        """
+        return self._resource.delete(self._basepath).expect_status(202).json
